function [simulation] = callWeMod2(simulation)
% Function callWeMod, wraps the simulation structure around weMod
% by converting back and from the simulation struct, and runs
% one step of the WeMod model
%
% Input:
% simulation - superstruct
%   .choke.Zc % nonlinear table of choke Areal 
%   .mudpump.flowRate - flowrate from mudmpump
%   .backppump.flowRate - flowrate backpressure pump
%   .model.rhod - drillstring density
%   .model.bVel - drillstring speed
%   .
%
% OutPut
%     simulation.model.BHP = y(7);
%     simulation.model.chokePressure = y(4);
%     simulation.model.rhoa = y(12);
%     
%     simulation.model.mudPumpPressure =  pressureProfileDrillString(1,1);
%     simulation.model.bitFlowRate = flow_bit;
%     simulation.model.chokeFlow = flow_choke ;
%     
%     simulation.model.qLiquid = 0;
%     simulation.model.qGas = 0;
%     simulation.model.alpha = 1;
%     simulation.model.massGas = 0;
%     simulation.model.betaa = simulation.model.betad * 0.5;
%     simulation.model.qbitDot = qbit - simulation.model.bitFlowRate;
%     
%     qbit = simulation.model.bitFlowRate;
% Written by Torbjrn Pedersen

%     Zc = simulation.choke.Zc;
%     Achoke = Zc*(6.77*0.0254); % Max 6.77 inches

    % We mod input
    % u1 - main pump rate, [m3/s] 
    % u2 - choke opening, [m2]..more than full opening...)
    % u3 - choke pump annulus pump rate, [m3/s] 
    % u4 - fluid density, [kg/m3](alternative: [SG] (relative to water density 1)
    % u5 - drillstring velocity  [m/s]
    % u6 - choke pump fluid density  [kg/m3]
    % u7 - annulus extraction pump flow rate  [m3/s]
    % u8 - annulus injection pump flow rate  [m3/s]  
    % u9 - annulus injection pump density  [kg/m3]
    % u10 - Drillstring Rotation. [RPM]
    
    persistent aChoke;
    
    if(isempty(aChoke))
        aChoke = 0.1;
    end
    
    u(1) = simulation.mudpump.flowRate; % Main pump rate [m3/s]
    u(2) = aChoke;
    u(3) = simulation.backppump.flowRate; % choke pump annulus pump rate [m3/s]
    u(4) = simulation.model.rhod; % fluid density [kg/m3]
    u(5) = simulation.model.bPosVel; %Drillstring velocity [m/s]
    u(6) = simulation.model.rhod;  % choke pump fluid density
    u(7) = 0; % annulus extraction pump flow rate [m3/s]
    u(8) = 0; % annulus injection pump flow rate [m3/s]
    u(9) = simulation.model.rhod; % annulus injection pump density [kg/m3]
    u(10) = 0;
    
    qref = simulation.controller.flowInput;
    ts = 1/(1/simulation.inc);
    
    simulation.weModSteps = 5;
    steps = simulation.weModSteps;
    
    for(cnt=1:1:steps)
        
    [simulation,y,depthProfileAnnulus,...
          verticalDepthProfileAnnulus,...
          pressureProfileAnnulus,...
          velocityProfileAnnulus,...
          densityProfileAnnulus,...
          temperatureProfileAnnulus,...
          massProfileAnnulus,...
          volumeProfileAnnulus,...
          depthProfileDrillString,...
          pressureProfileDrillString,...
          velocityProfileDrillString,...
          densityProfileDrillString,...
          temperatureProfileDrillString,...
          massProfileDrillString,...
          volumeProfileDrillString] = stepWeMod2(u,simulation);
    
    q = simulation.model.chokeFlow;
    %u(2) = innerFlowControl(qref,q,ts); % choke opening [% of m2] 
    u(2) = innerFlowControl2(qref,simulation); % choke opening [% of m2] 
%     dp = max(simulation.model.chokePressure - simulation.model.patm,0.001);
%     u(2) = qref / (0.004*sqrt((2/1180)*dp));
%     u(2) = max(u(2),0.005);
%     u(2) = min(u(2),0.185);
      
    end
    
    simulation.model.betaa = simulation.model.bulkchange*simulation.model.betad;
    aChoke = u(2);
    
    WeMod.depthP = depthProfileAnnulus;
    WeMod.verDepthP = verticalDepthProfileAnnulus;
    WeMod.presP = pressureProfileAnnulus;
    WeMod.velP = velocityProfileAnnulus;
    WeMod.denP = densityProfileAnnulus;
    WeMod.tempP = temperatureProfileAnnulus;
    WeMod.massP = massProfileAnnulus;
    WeMod.volP = volumeProfileDrillString;
    
    simulation.WeMod = WeMod;
    simulation.controller.aChoke = aChoke;
    
    % WeMod point data
    % y(1) - stand pipe pressure
    % y(2) - Pressure before bit
    % y(3) - Pressure after bit
    % y(4) - Choke pressure
    % y(5) - Liquid level drill string
    % y(6) - Liquid level annulus
    % y(7) - Bottom hole pressure
    % y(8) - Bit position
    % y(9) - Total measured depth
    % y(10) - Return volume flow rate
    % y(11) - Density in
    % y(12) - Density out
    % y(13) - Choke opening
    % y(14) - Drill string position
    % y(15) - Drill string length

%     persistent qbit;
    
%     in2m=0.0254;
%     Abit = 1*in2m;
%     Achoke = 6.77 * in2m;
% 
%     v_bit = - velocityProfileAnnulus(end,end);
%     flow_bit = (Abit * v_bit);
%     
%     v_choke = - velocityProfileAnnulus(end,1);
%     flow_choke = (Achoke * v_choke);
%     
%     if(isempty(qbit))
%         qbit = flow_bit;
%     end   
%     
%     simulation.model.BHP = y(7);
%     simulation.model.chokePressure = y(4);
%     simulation.model.rhoa = y(12);
%     
%     simulation.model.mudPumpPressure =  pressureProfileDrillString(1,1);
%     simulation.model.bitFlowRate = flow_bit;
%     simulation.model.chokeFlow = flow_choke ;
%     
%     simulation.model.qLiquid = 0;
%     simulation.model.qGas = 0;
%     simulation.model.alpha = 1;
%     simulation.model.massGas = 0;
%     simulation.model.betaa = simulation.model.betad * 0.5;
%     simulation.model.qbitDot = qbit - simulation.model.bitFlowRate;
%     
%     qbit = simulation.model.bitFlowRate;
 
end