function u = controllerMRACflow(simulation)
% Simple choke controller, inverts choke flow equation to get
% correct choke opening, uses a simple search algorithm if the
% flow is nonlinear
%
% Inputs:
% simulation - superstruct
%   .simulation.controller.flowInput - u_c,ref, wanted choke flow
%   .simulation.controller.chokeReference - p_c,ref wanted choke pressure
%   .simulation.controller.patm - atmospheric pressure
%   .simulation.model.rhoa - annulus density
%   .simulation.model.Kc - choke gain
%
% Output:
% u - z_c,ref wanted choke opening
%
% Written by Torbjrn Pedersen

u_c0 = simulation.controller.flowInput;

% p_c = simulation.measurement.chokePressure;
p_c0 = simulation.controller.chokeReference;

p_atm = simulation.model.patm;
rho = simulation.model.rhoa;
Kc = simulation.choke.Kc;
flowTable = simulation.choke.flowTable;

% dP = max(0.1*10^5,(p_c-p_atm));
dP = max(0.1*10^5,(p_c0 - p_atm)); % Min pressure drop 0.1 bar

if(simulation.disturbance.type.nonlinearChoke)
    Zc = u_c0 / (Kc*sqrt((2/rho)*dP));
    if(Zc > 1)
        u = 1;
        return;
    elseif(Zc < 0)
        u = 0;
        return;
    end

    if(flowTable(floor(Zc*1000)+1)==0)
        Zc = 0;
    else
        ZcTemp = u_c0 / (Kc*sqrt((2/rho)*dP));
        inc = 0.05;
        running = 1;
        
        while(running)
            
            temp = Kc*ZcTemp*sqrt((2/rho)*(dP));
            temp = temp*flowTable(floor(ZcTemp*1000)+1);           
           
            if(abs(temp - u_c0) < 0.001)
                running = 0;
                Zc = ZcTemp; 
            elseif(temp < u_c0)
                ZcTemp = ZcTemp + inc;
                inc = inc*2;
            elseif(temp > u_c0)
                inc = inc/2;
                ZcTemp = ZcTemp - inc;
                inc = inc/2;
            end
            
            if(ZcTemp > 1)
                Zc = 1;
                running = 0;
            end            
            
        end
            
        
    end
else
    Zc = u_c0 / (Kc*sqrt((2/rho)*dP)); % Inverted choke equation
end

if Zc > 1
    Zc = 1;
end

if Zc < 0
    Zc = 1;
end

u = Zc;

end 