function [simulation] = controllerPIchoke(simulation)
% controller - gives the desired control input for the choke
% 
% simulation = controller(simulation) 
%       
% INPUTS
% simulation - superstruct
%   .controller.integralError - Previous integral error
%   .controller.epsilon1       - Controller deadband inner
%   .controller.epsilon2       - Controller deadband outer
%   .controller.error  - r - p_c
%   .controller.previousChokeControlInput - last input used
%   .num                      - numerical timestep size
%
% OUTPUTS
% simulation - superstruct
%   .controller.integralError - New integral error
%   .controller.previousChokeControlInput - Updated prev input
%   .controller.chokeInput - New choke input
%
% Written by Torbjrn Pedersen 2008

%Get the current integral error
ie = simulation.controller.integralError;
ieTemp = ie;
%Get deadband
persistent triggerDeadband;

if isempty(triggerDeadband)
     triggerDeadband = 0;
end

deadbandI = simulation.controller.epsilon1;
deadbandO = simulation.controller.epsilon2;
%Get numerical step
dT = simulation.inc;

%Find the error
error = simulation.controller.error; 

%Stop controller if within deadband or if samplingFalse;
if (abs(error) < deadbandI);
    simulation.controller.chokeInput = simulation.controller.previousChokeControlInput;
    triggerDeadband = 1;
    return;
end

if (abs(error) < deadbandO) && (triggerDeadband);
    simulation.controller.chokeInput = simulation.controller.previousChokeControlInput;
    return;
elseif(triggerDeadband)
    triggerDeadband = 0;
end

%the controller parameters
Kp2 = simulation.controller.Kp2;
Ti2 = simulation.controller.Ti2;


if (simulation.controller.enablePI) % other -> Pi control (reset error,set bias, reset flag)
    ie = 0; % Reset error
    simulation.controller.ucb = simulation.controller.chokeInput; % Set bias
    simulation.controller.enablePI = 0; % reset flag
    uc = simulation.controller.ucb + Kp2*error; % new uc
elseif (Ti2 == 0) % integral effect off
    uc = simulation.controller.ucb + Kp2*error;    
else
    ie = ie + dT*error;     %Integral (sum) action
    uc = simulation.controller.ucb + Kp2*(error + (1/Ti2)*ie);
end


%Take care of saturation and prevent wind-up
if uc <= 0
    uc = 0;
    ie = ieTemp;
elseif (uc) >= 1
    uc = 1;
    ie = ieTemp;
end

simulation.controller.integralError = ie;
simulation.controller.previousChokeControlInput = uc;
simulation.controller.chokeInput = uc;
end
