function [simulation] = controllerPIflowBHP(simulation)
% controller - gives the desired control input for the choke
% 
% simulation = controller(simulation) 
%       
% INPUTS
% simulation - superstruct
%   .controller.integralError - Previous integral error
%   .controller.epsilon       - Controller deadband parameters
%   .controller.error - r - p_c
%   .controller.previousChokeControlInput - last input used
%   .measurement.Pbit         - "Measured" bit pressure
%   .num                      - numerical timestep size
%
% OUTPUTS
% simulation - superstruct
%   .controller.integralError - New integral error
%   .controller.previousChokeControlInput - Updated prev input
%   .controller.flowInput - New choke input
%
% Written by Torbjrn Pedersen 2008

%Get the current integral error
ie = simulation.controller.integralErrorFlow;
ieTemp = ie;
%Get deadband
persistent triggerDeadband;

if isempty(triggerDeadband)
     triggerDeadband = 0;
end

deadbandI = simulation.controller.epsilon1;
deadbandO = simulation.controller.epsilon2;
%Get numerical step
dT = simulation.inc;

%Find the error
error = simulation.controller.error; 

%Stop controller if within deadband or if samplingFalse;
if (abs(error) < deadbandI);
    simulation.controller.flowInput = simulation.controller.previousFlowControlInput;
    triggerDeadband = 1;
    return;
end

if (abs(error) < deadbandO) && (triggerDeadband);
    simulation.controller.flowInput = simulation.controller.previousFlowControlInput;
    return;
elseif(triggerDeadband)
    triggerDeadband = 0;
end

%the controller parameters
Kp2 = simulation.controller.PIflowKp;
Ti2 = simulation.controller.PIflowTI;


if (simulation.controller.enablePIflow) % other -> Pi control (reset error,set bias, reset flag)
    ie = 0; % Reset error
    simulation.controller.ucbFlow = simulation.controller.flowInput; % Set bias
    simulation.controller.enablePIflow = 0; % reset flag
    uc = simulation.controller.ucbFlow + Kp2*error; % new uc
elseif (Ti2 == 0) % integral effect off
    uc = simulation.controller.ucbFlow + Kp2*error;    
else
    ie = ie + dT*error;     %Integral (sum) action
    uc = simulation.controller.ucbFlow + Kp2*(error + (1/Ti2)*ie);
end


%Take care of saturation and prevent wind-up
if uc <= 0
    uc = 0;
    ie = ieTemp;
elseif (uc) >= 15000/60/1000
    uc = 15000/60/1000;
    ie = ieTemp;
end

simulation.controller.integralErrorFlow = ie;
simulation.controller.previousFlowControlInput = uc;
simulation.controller.flowInput = uc;
end
