function [qchoke, qGas, qLiq,Achoke] = getChokeFlowRates(simulation,P1,P2,alpha,rhoGas,rhoLiq)
% [qchoke, qGas, qLiq] = getChokeFlowRates(self,P1,P2,alpha,rhoGas,rhoLiq)
%
% Returns the total flow, gas flow and liquid flow through a choke.
%
%
% Outputs:
% qchoke- volume flow rate [m3/s]
% qGas  - gas volume flow rate [m3/s]
% qLiq  - liquid volume flow rate [m3/s]
%
% Inputs:
% simulation- superstruct
% P1        - Upstream pressure [barA]
% P2        - Downstream pressure [barA]
% alpha     - mass fraction of liquid (0 - 1, where 1 is pure liquid)
% rhoGas    - Gas Density at 1 bar and 15 degC [kg/m3]
% rhoLiq    - Liquid Density at 1 bar and 15 degC [kg/m3]
%
% See [Fjalsestad et.al.] for more information about 2ph chokes
%
% Written by Eirik Opsanger / Torbjrn Pedersen 2008

Kc = simulation.choke.Kc;
Zc = simulation.choke.Zc;
flowTable = simulation.choke.flowTable;

xT = 0.5;                           % Pressure ratio limit (~0.5)
Fk = 1;                             % Gas type compensation (typ. 1)
P1 = max(P1,1.01*10^5);                  % Avoid dividing by zero
x  = min((P1-P2)/P1,xT);            % Pressure ratio
Y  = 1 - x/(3*Fk*xT);               % Gas expansion factor
Y  = Y*x*P1/(P1-P2);                % Adjust for sonic flow


% Stratified flow (See [Fjalestad et.al.] for more information)
w = Zc*Kc*Y*sqrt(2*(P1-P2)*rhoGas)/...
        (1+alpha*(Y*sqrt(rhoGas/rhoLiq)-1));

w_gas = w*(1-alpha);
w_liq = w*alpha;

% Volume flow rates [m3/s]
qchoke  = w_gas/rhoGas + w_liq/rhoLiq;
qGas   = w_gas/rhoGas;
qLiq   = w_liq/rhoLiq;

%qGas = 0;
%qLiq = 0;
%qchoke = Kc*Zc*sqrt((2/rhoLiq)*(P1-P2));

% Nonlinear table lookup
qchoke = qchoke*flowTable(floor(Zc*1000)+1);


return