function [flowTable,areaTable] = makeChokeChar(simulation)
% This function generates a flowTable with 1000 elements, each element
% gives the percentage of flow through the valve versus linear flow
%
% Output:
% FlowTable - table with real flow in percent versus linear flow
%
% Torbjrn Pedersen

CfUSGPM2CMPS = (3.785411784 * 10^-3)/60; % Conversion factor US Gallons per min to cubic metres per sec
CfCI2CM = 0.0254^2; % Conversion factor, cubic inches to cubic metres
dimensionFactor = 0.66; % Size compared to 3 inch valve
elements = 1000; % number of elements in the flowtable

% Choke char matrix
% [Pct open, diameter inch, areal, cv , equivbean]
% Pct open - opening of choke in percent
% diameter - diameter of the opening in inches
% areal - areal of the opening in inches
% Cv - % Cv (flow coefficient) is defined as the volume flow rate of water at standard conditions 
%   that passes through the valve (in USGPM) when a pressure drop of 1 psi
%   is across the valve.
%   Q = Cv * sqrt(dp/G);
% equivbean - Equivalenet bean
%   the bean is the restriction in a choke valve.  measured in
%   64ths ion and inch, bigger bean -> bigger flow
Cm = [ 0, 0, 0, 0, 0;
    10, 0.156, 0.019, 0.58, 10;
    20, 0.156, 0.019, 0.58, 10;
    30, 0.781, 0.479, 14.38, 50;
    40 1.438 1.623 48.69 92;
    50 1.719 2.320 69.60 110;
    60 2.000 3.142 94.25 128;
    70 2.250 3.976 119.28 144;
    80 2.516 4.970 149.11 161;
    90 2.781 6.075 182.26 178;
    100 2.938 6.777 203.31 188;];

X = Cm(:,1)/100; % Using valve opening
Y = Cm(:,4).*CfUSGPM2CMPS*dimensionFactor*3.33; %/0.0085; % and scaled flow coefficient
Y2 = Cm(:,3).*CfCI2CM*dimensionFactor*3.33;

XX = 0:1/elements:1; % Range

% Create flowTable by using linear interpolation across scaled Cv
CVFlowTable = interp1(X,Y,XX);
CVAreaTable = interp1(X,Y2,XX);

% Choke settings used to generate 1 bar drop choke response
simulation.choke.ZcVel = 0; 
simulation.controller.chokeInput = 0;
simulation.timeSpan = 0:0.1:1;
simulation.choke.Zc = 0;

Pc = 2*10^5;
patm = 1*10^5;
alpha = 1; 
rhogOut = 1; %*10^-3;
rhol = 1000; %*10^-3;
qchoke = zeros(11,1);
k = 1;

for i = 0:0.1:1;
% Increase input and initalize opening
simulation.controller.chokeInput = i;
simulation.choke.Zc = i;

% Get the choke response and flow
simulation = getChoke(simulation);
qchoke(k,1) = getChokeFlowRatesLinear(simulation,Pc,patm,alpha,rhogOut,rhol);

% Update index
k = k + 1;
end

% Scaled choke flow response
qchokeS = qchoke; %./ 250; %;./0.0075;

% Create chokeFlowTable by linear interpolation
chokeFlowTable = interp1(X,qchokeS,XX);

% Genearte percent flow table based on CV response and choke response
percentFlow =  CVFlowTable ./ chokeFlowTable;
percentFlow(1) = 0; % remove index with bad division

% Calculate back flow to check table
Flow = percentFlow.*chokeFlowTable;

figure(1)
hold on
%plot(XX,Flow); % Plot flow
plot(XX,chokeFlowTable,'r'); % Plot Choke Flow Table
plot(XX,CVFlowTable,'g--');  % Plot CV Flow Table
plot(XX,percentFlow,'b')
hold off
legend('Choke model flow','Real choke flow','Lookup table');
xlabel('Choke Opening [%]')

% figure(2)
% plot(XX,CVAreaTable)

% Return flowtable
flowTable = percentFlow;
areaTable = CVAreaTable;

end
