function [ simulation ] = moveDrillPipe(simulation)
%MOVEDRILLPIPE moves the drillpipe if bPosVel~=0
%
%   simulation = moveDrillPipe(simulation) integrates
%   bPosVel to find new bPos
%   
%   Input:
%   simulation - superstruct
%       .model.LwCurrent - current length of well
%       .model.Ld  - length vector of drilling equipment/string
%       .model.Lw  - length vector of well
%       .model.bPos - current block position
%       .model.bPosVel - block position velocity 
%       .model.pipeStandLength - standard stand length
%       
%   Output:
%   simulation - superstruct
%       .model.lw - new length
%       .model.bPos - new blockposition
%
% Torbjrn Pedersen / yvind Stamnes 2008

LwC = simulation.model.LwCurrent;
Ld  = simulation.model.Ld;
Lw  = simulation.model.Lw;
lBit = getBitPosition(simulation); 
Lw(end);

timeSpan = simulation.timeSpan;

% initial condition
bp0 = simulation.model.bPos;

% get velocity
bPosVel = simulation.model.bPosVel;
  
% get standard pipeStandLength
psl = simulation.model.pipeStandLength;

% move it
if LwC(end)==Lw(end) && bPosVel<0 && (lBit == Lw(end))
    disp('Maximum well length reached, cannot drill further')
    bp=bp0;
else
    bpArray = ode1(@(t,y)solveBlockPos(t,y,bPosVel),timeSpan,bp0);
    bp = bpArray(end);
end

if bPosVel==0
    % no speed, no movement, do nothing
    return
end

% if block pos is smaller than zero simply reset to zero, and add new
% pipe if drilling down
bp = max(0,bp);

                 
% if block pos is larger than one pipe stand length reset
bp = min(psl,bp);

% if pipe is moving into the well
if bPosVel<0
    % shift to LwC reference frame
    LdShifted = Ld - bp;

    % if bit is on bottom and drilling has occured
    if LwC(end)<LdShifted(end) 
        if LdShifted(end)<=Lw(end)
            %ordinary drilling
            % set new current length of well
            simulation.model.lw = LdShifted(end);
        else 
            % cannot drill further
            simulation.model.lw = Lw(end);
            bp = (Ld(end)-Lw(end));
        end
    end
end

% set blockposition
simulation.model.bPos = bp;

if (bp == 0) && (bPosVel < 0) && (Ld(end)<Lw(end))
      simulation = addDrillPipeStand(simulation);
end

return

function dydt= solveBlockPos(t,y,bpv)
dydt = bpv;
return

