function qres = simpleReservoirModel(simulation)
% Simple reservoir model which calculates reservoir flow based on
% bit, pore and fraction pressures.
%
% Input: 
% simulation - superstruct
%   .model.BHP - bit pressure
%   .reservoir.p_pore - pore pressure
%   .reservoir.p_frac - fraction pressure
%
% OutPut:
% qres - reservoir flow
%
% Written by Torbjrn Pedersen 2008


    % if the reservoir flow is disable set qres to zero and return
    if(~simulation.disturbance.qresEnabled)
        qres = simulation.model.qres;
        return;
    end

    % get reservoir inputs
    qres = simulation.model.qres;
    p_pore = simulation.reservoir.porePressure;
    p_frac = simulation.reservoir.fracPressure;
    
    % get bit pressure
    p_bit = simulation.model.BHP;

    % Influx (kick) case, if below pore pressure calculate influx
    if (p_pore > p_bit)
        kickConstant = simulation.reservoir.kickConstant;
        qres = qres + (p_pore - p_bit)*kickConstant;
    
    % Loss case 1: if over fraction pressure calculate loss   
    elseif (p_bit > p_frac)
        mudLossConstant = simulation.reservoir.mudLossConstant;
        qres = qres + (p_frac - p_bit)*mudLossConstant;
        
        % If contionous loss add extra diff
        if(simulation.disturbance.continousMudLossEnabled)
            continousMudLossConstant = simulation.reservoir.continousMudLossConstant;
            qres = qres - abs(p_bit - p_pore)*continousMudLossConstant;
        end
        
    % case continous loss inside boundry   
    elseif simulation.disturbance.continousMudLossEnabled
        continousMudLossConstant = simulation.reservoir.continousMudLossConstant;
        qres = qres - abs(p_bit - p_pore)*continousMudLossConstant;        
    
    % Inside boundry case and no contionous loss   
    else
        qres = 0;
      
    end
       
    return; 
end