function simulationStructure()
%% This function runs a simulation of the system using settings and
% disturbances as defined by simulationInit()
clear all, close all hidden, clc;

%Save information
id = datestr(now);
structure = 'Top';
info = 'Con';

weModCase = 6;

tic;

simulation = simulationInit();
% Simulation init holds the settings for simulation length and
% parameters

controller = simulation.controller.type;

decimation = max(1,(1/simulation.inc)/10);
% Initalize Data vectors 
BHA = zeros(simulation.endtime*(1/(decimation*simulation.inc))-1,4);
Choke = zeros(simulation.endtime*(1/(decimation*simulation.inc))-1,6);
MudPump = zeros(simulation.endtime*(1/(decimation*simulation.inc))-1,6);
BackPressurePump = zeros(simulation.endtime*(1/(decimation*simulation.inc))-1,4);
Position = zeros(simulation.endtime*(1/(decimation*simulation.inc))-1,4);
Controller = zeros(simulation.endtime*(1/(decimation*simulation.inc))-1,16);
Well = zeros(simulation.endtime*(1/(decimation*simulation.inc))-1,12);
Reservoir = zeros(simulation.endtime*(1/(decimation*simulation.inc))-1,3);
Observer = zeros(simulation.endtime*(1/(decimation*simulation.inc))-1,7);

if simulation.useWeMod
    WeMod = zeros(simulation.endtime,8);
	initWeMod(weModCase);  % Init WeMod Case 6 Kristin
end

load 'input_U'; % Pregenerated choke and pump vectors

index = 1;

% Simulate system for simulation time
while(simulation.time <= simulation.endtime)
    
    
    % Update simulation timespan
    simulation.timeSpan = simulation.time:simulation.num:(simulation.time+simulation.inc);
    simulation.timeOde = [simulation.time, simulation.time+simulation.inc];
 
    % Calculate disturbances
    simulation = calculateDisturbance(simulation,index);
    
    % Calculate choke opening from input and dynamics
    if(~simulation.controller.idealFlow)
    simulation = getChoke(simulation);
    end
    
    %Use a standard input vector
    if simulation.input.chokeVector 
        simulation.choke.Zc = U.c(simulation.time)/100;
    end
    
    % Calculate pump flows
    if(~mod(index,simulation.updatemodel) || index == 1)
    simulation.timeSpan2 = simulation.time:simulation.modelnum:(simulation.time+(simulation.updatemodel*simulation.inc));
    simulation.mudpump  = getPump(simulation.mudpump,simulation.timeSpan2);
	simulation.backppump = getPump(simulation.backppump,simulation.timeSpan2);
    end
    
    % Use a standard input vector
    if simulation.input.pumpVector 
    	simulation.mudpump.flowRate = U.p(simulation.time)/(60*1000);
    end
   
    % Increase flow from backpressure pump if main pump input is below 400
    % l/min
    if(simulation.mudpump.flowRate < 400/60/1000)
        simulation.backppump.input = 1;
    else
        simulation.backppump.input = 0.5;
    end
    
    % solve hydraulic model for one time step
    if(~mod(index,simulation.updatemodel) || index == 1)
        
    if(simulation.useWeMod)
        simulation = callWeMod2(simulation); % With weModForMatlab
    else
       if(simulation.controller.idealFlow)
           if(simulation.disturbance.type.dynamics)
                simulation = solveHydraulicModelFlow(simulation); % With flow controller
           else
                simulation = solveHydraulicModelFlowNoDyn(simulation);
           end
       else
            simulation = solveHydraulicModelChoke(simulation); % With choke controller
       end
    end
    end

    % "take" measurements from system
    simulation = takeMeasurement(simulation,index);

    % Update data vectors
    %inct = (1/simulation.inc)
    %index = simulation.time*inct
    %test = index - 3
    if(~mod(index,decimation))
    
    BHA(index/decimation,1) = simulation.model.BHP*10^-5;
    BHA(index/decimation,2) = simulation.measurement.BHP*10^-5;
    BHA(index/decimation,3) = simulation.model.bitFlowRate;
    BHA(index/decimation,4) = simulation.measurement.bitFlowRate;
    Choke(index/decimation,1) = simulation.model.chokePressure*10^-5;
    Choke(index/decimation,2) = simulation.measurement.chokePressure*10^-5;
    Choke(index/decimation,3) = simulation.model.chokeFlow;
    Choke(index/decimation,4) = simulation.measurement.chokeFlow;
    Choke(index/decimation,5) = simulation.choke.Zc;
    Choke(index/decimation,6) = simulation.controller.chokeInput;
    MudPump(index/decimation,1) = simulation.model.mudPumpPressure*10^-5;
    MudPump(index/decimation,2) = simulation.measurement.mudPumpPressure*10^-5;
    MudPump(index/decimation,3) = simulation.mudpump.flowRate;
    MudPump(index/decimation,4) = simulation.measurement.mudPumpFlow;
    MudPump(index/decimation,5) = simulation.mudpump.rotationalSpeed;
    MudPump(index/decimation,6) = simulation.mudpump.input;
    BackPressurePump(index/decimation,1) = simulation.backppump.flowRate;
    BackPressurePump(index/decimation,2) = simulation.measurement.backPPumpFlow;
    BackPressurePump(index/decimation,3) = simulation.backppump.rotationalSpeed;
    BackPressurePump(index/decimation,4) = simulation.backppump.input;
    Position(index/decimation,1) = simulation.model.hbit;
    Position(index/decimation,2) = simulation.model.lw;
    Position(index/decimation,3) = simulation.model.bPos;
    Position(index/decimation,4) = simulation.model.bPosVel;
    Well(index/decimation,2) = simulation.model.Md;
    Well(index/decimation,1) = simulation.model.Ma;
    Well(index/decimation,3) = simulation.model.Fd;
    Well(index/decimation,4) = simulation.model.Fa;
    Well(index/decimation,5) = simulation.model.Va;
    Well(index/decimation,6) = simulation.model.Vd;
    Well(index/decimation,7) = simulation.model.betaa;
    Well(index/decimation,8) = simulation.model.betad;
    Well(index/decimation,9) = simulation.model.massGas;
    Well(index/decimation,10) = simulation.model.rhoa;
    Well(index/decimation,11) = simulation.model.rhod;
    Well(index/decimation,12) = simulation.model.VaDot;
    Reservoir(index/decimation,1) = simulation.model.qres;
    Reservoir(index/decimation,2) = simulation.reservoir.porePressure;
    Reservoir(index/decimation,3) = simulation.reservoir.fracPressure;
    
    end
    
    if(simulation.useWeMod)
        WeMod.depthP(:,index) = simulation.WeMod.depthP;
        WeMod.verDepthP(:,index) = simulation.WeMod.verDepthP;
        WeMod.presP(:,index) = simulation.WeMod.presP;
        WeMod.velP(:,index) = simulation.WeMod.velP;
        WeMod.denP(:,index) = simulation.WeMod.denP;
        WeMod.tempP(:,index) = simulation.WeMod.tempP;
        WeMod.massP(:,index) = simulation.WeMod.massP;
        WeMod.volP(:,index) = simulation.WeMod.volP;
    end
    
    % Update observer
    if(~mod(index,simulation.updatemodel))
        simulation = observer(simulation);
    end
    
    if(~mod(index,decimation))
        Observer(index/decimation,1) = simulation.observer.Fa;
        Observer(index/decimation,2) = simulation.observer.rhoa;
        Observer(index/decimation,3) = simulation.observer.qbit;
        Observer(index/decimation,4) = simulation.observer.pbit;
        Observer(index/decimation,5) = simulation.observer.xi;
        Observer(index/decimation,6) = simulation.observer.zetahat1;
        Observer(index/decimation,7) = simulation.observer.zetahat2;
    end
    
    
    % Update simulation error first time
    if isempty(simulation.controller.error)
        simulation.controller.error =  simulation.measurement.BHP*10^-5 - simulation.controller.PbitReference;
    end
    
    
    
    % calculate next controller output 
    simulation = controlSystem(simulation, index);

    % Save controller data
    if(~mod(index,decimation))
    Controller(index/decimation,1) = simulation.controller.error;
    Controller(index/decimation,2) = simulation.controller.PbitReference;
    Controller(index/decimation,3) = simulation.controller.chokeReference;
    Controller(index/decimation,4) = simulation.controller.gain(1);
    Controller(index/decimation,5) = simulation.controller.gain(2);
    Controller(index/decimation,6) = simulation.controller.gain(3);
    Controller(index/decimation,7) = simulation.refmod.chokePressure;
    Controller(index/decimation,8) = simulation.controller.flowInput;
    Controller(index/decimation,9) = simulation.l1.l1parameters(1);
    Controller(index/decimation,10) = simulation.l1.l1parameters(2);
    Controller(index/decimation,11) = simulation.l1.l1parameters(3);
    Controller(index/decimation,12) = simulation.l1model.pbit;
    Controller(index/decimation,13) = simulation.controller.unfilteredu;
    Controller(index/decimation,14) = simulation.controller.l1X;
    Controller(index/decimation,15) = simulation.l1model.pchoke;
    Controller(index/decimation,16) = simulation.controller.aChoke;
    end
    
    % Movement model
    
    % Move drillstring due to drilling or disturbances (heave,
    % connection..)
    
    if(~mod(index,simulation.updatemodel))
    simulation = moveDrillPipe(simulation); 
    
    % Update the geometry of the well based on new length to bit
    simulation = updateCurrentGeometry(simulation);
    simulation = updateGeometry(simulation);
    
    % Update reservoir flow
    simulation.model.qres = simpleReservoirModel(simulation);
    end
    
    
    % Update simulation time
    simulation.time = simulation.time + simulation.inc;
    
    if(~mod(index,(0.1*60*10)))
        clc
        disp('time:')
        disp(floor(simulation.time))
        disp('endtime:')
        disp(simulation.endtime)
    end
    
        
    index = index + 1;
   
end

Time.telapsed = toc
Time.decimation = decimation;
Time.inc = simulation.inc;

% Clean-up WeMod 
if(simulation.useWeMod)
	 endWeMod(weModCase);
end

% id = strrep( id,':', '');
% id = strrep( id,'-', '');
% id = strrep( id,' ', '-');
% dirname = strcat(id,'-',controller,structure,info);

%mkdir(strcat('C:\Torbjp\MSc\Code\Simulator\',dirname))

% Save data vectors
% if(~simulation.useWeMod)
% copyfile('simulationInit.m',strcat('.\',dirname,'\','init.m'))
% copyfile('calculateDisturbance.m',strcat('.\',dirname,'\','disturbance.m'))
% save('Data_Time','Time')
% save('Data_BHA','BHA')
% save('Data_Choke','Choke')
% save('Data_MudPump','MudPump')
% save('Data_BackPressurePump','BackPressurePump')
% save('Data_Position','Position')
% save('Data_Controller','Controller')
% save('Data_Well','Well')
% save('Data_Reservoir','Reservoir')
% save('Data_Observer','Observer')
% end
save Data_Time 'Time';
save Data_BHA 'BHA';
save Data_Choke 'Choke';
save Data_MudPump 'MudPump';
save Data_BackPressurePump 'BackPressurePump';
save Data_Position 'Position';
save Data_Controller 'Controller';
save Data_Well 'Well';
save Data_Reservoir 'Reservoir';
save Data_Observer 'Observer';

if(simulation.useWeMod)
% save(strcat('C:\Torbjp\MSc\Code\Simulator\',dirname,'\','Data_WeMod'),'WeMod')
% copyfile('C:\Torbjp\MSc\Code\Simulator\simulationInit.m',strcat('C:\Torbjp\MSc\Code\Simulator\',dirname,'\','init.m'))
% copyfile('C:\Torbjp\MSc\Code\Simulator\calculateDisturbance.m',strcat('C:\Torbjp\MSc\Code\Simulator\',dirname,'\','disturbance.m'))
% save(strcat('C:\Torbjp\MSc\Code\Simulator\',dirname,'\','Data_Time'),'Time')
% save(strcat('C:\Torbjp\MSc\Code\Simulator\',dirname,'\','Data_BHA'),'BHA')
% save(strcat('C:\Torbjp\MSc\Code\Simulator\',dirname,'\','Data_Choke'),'Choke')
% save(strcat('C:\Torbjp\MSc\Code\Simulator\',dirname,'\','Data_MudPump'),'MudPump')
% save(strcat('C:\Torbjp\MSc\Code\Simulator\',dirname,'\','Data_BackPressurePump'),'BackPressurePump')
% save(strcat('C:\Torbjp\MSc\Code\Simulator\',dirname,'\','Data_Position'),'Position')
% save(strcat('C:\Torbjp\MSc\Code\Simulator\',dirname,'\','Data_Controller'),'Controller')
% save(strcat('C:\Torbjp\MSc\Code\Simulator\',dirname,'\','Data_Well'),'Well')
% save(strcat('C:\Torbjp\MSc\Code\Simulator\',dirname,'\','Data_Reservoir'),'Reservoir')
	 save Data_WeMod 'WeMod';
end

testplot();

end