function [simulation] = solveHydraulicModelFlow(simulation)
% SOLVEHYDRAULICMODEL - solves the hydralic ODE model of the system.
%
% simulation = solveHydraulicModel(simulation)
%          
%
% INPUTS 
% simulation - Structure with simulation parameters 
%     .model.betad  - constants See [Kassa2007] for definitions
%     .model.betag
%     .model.Vd
%     .model.Vd;
%     .model.Va;
%     .model.VaDot;
%     .model.Md;
%     .model.Ma;
%     .model.Fd;
%     .model.Fa;
%     .model.rhod;
%     .model.rhogIn;
%     .model.rhogOut;
%     .model.rhoaChange;
%     .model.hbit;
%     .model.g;
%     .model.patm;
%     .model.qRes;
%     .model.mudPumpPressure; - Previous pressure in mudpump
%     .model.bitFlowRate;     - Previous bit flowrate
%     .model.chokePressure;   - Previous choke pressure
%     .model.massOfGas;       - Previous mass of gas
%     .mudpump.Kp             - Mudpump gain
%     .mudpump.input;         - Mudpump control input
%     .backppump.Kb           - Backpressurepump gain
%     .backppump.input;       - Backpressurepump control input
%
% OUTPUTS
% simulation - Updated simulation structure
%     .model.mudPumpPressure  - New mud pump pressure
%     .model.bitFlowRate      - New bit flow rate
%     .model.chokePressure    - New choke pressure
%     .model.massGas          - New mass of gas
%     .model.rhoa             - Calculated density in annulus
%     .model.betaa            - Calculated bulk modulus in annulus
%     .model.alpha            - mass fraction of liquid
%     .model.chokeFlow        - New choke flow
%     .model.qLiquid          - New liquid flow
%     .model.qGas             - New gas flow
%     .model.BHP              - New bit pressure
%
% Written by Torbjrn Pedersen 2008

%Get constants (See [Kassa2007] for definitions)
betad   = simulation.model.betad;
betag   = simulation.model.betag;
Vd      = simulation.model.Vd;
Va      = simulation.model.Va;
VaDot   = simulation.model.VaDot;
Md      = simulation.model.Md;
Ma      = simulation.model.Ma;
Fd      = simulation.model.Fd;
Fa      = simulation.model.Fa;
rhod    = simulation.model.rhod;
rhogIn  = simulation.model.rhogIn;
rhogOut = simulation.model.rhogOut;
rhoaChange = simulation.model.rhoaChange;
hbit    = simulation.model.hbit;
g       = simulation.model.g;
patm    = simulation.model.patm;
M = Ma+Md;

%In-/outflows Qresorvoir, Qmudpump, QBackpressurepump
qres    = simulation.model.qres;
qpump   = simulation.mudpump.flowRate;
qback   = simulation.backppump.flowRate;

%Initial values mudpump pressure, bitflow, chokepressure and mass of gas
Pp0     = simulation.model.mudPumpPressure;
qbit0   = simulation.model.bitFlowRate;
Pc0     = simulation.model.chokePressure;
mg0     = simulation.model.massOfGas;
% qc0     = simulation.idealFlow.flow;
% qcDot0  = simulation.idealFlow.flowChange;

%y0 = [Pp0 qbit0 Pc0 mg0 qc0 qcDot0]';
y0 = [Pp0 qbit0 Pc0 mg0]';

% Solve system one timeSpan
[y, varargout] = ode4m(@(t,y)defineSystem(t, y, simulation, Vd, betad, Va, VaDot, betag, Ma, Md, Fd, Fa, rhod, rhogIn, rhogOut, rhoaChange, qpump, qback, qres, g, hbit, patm),simulation.timeSpan2,y0);

%Get new values for the states
Pp  = max(patm,y(end,1)); % Pump pressure can not be less than 0.
qbit= max(0,y(end,2)); % qbit can not be less than 0, due to one way valve.(assumption in model)
Pc  = max(patm,y(end,3)); % Pc can not be less than 1 (atmospheric pressure)
mg  = max(0,y(end,4)); % Mass of Gas can not be less than 0.
qc  = simulation.controller.flowInput;
qcDot = 0;

%The additional values from defineSystem(...)
rhoa    =  varargout(end,1).rhoa;
betaa   =  varargout(end,1).betaa;
alpha   =  varargout(end,1).alpha;
qLiquid =  varargout(end,1).qLiq;
qGas    =  varargout(end,1).qGas;
qbitDot =  varargout(end,1).qbitDot;


%Return the states
simulation.model.BHP                = Pc + Ma*qbitDot + Fa* abs(qbit+qres)*(qbit+qres) + rhoa * g * hbit;
% simulation.model.BHP                = Pp - Md*qbitDot - Fd* abs(qbit+qres)*(qbit+qres) + rhod * g * hbit;
% simulation.model.BHP                = Pc - Ma*qbitDot - Fa*(qbit+qres)^2 + rhoa * g * hbit;
% simulation.model.BHP                = (Pc*Md + Pp*Ma + (Md*Fa-Ma*Fd)*abs(qbit+qres)*(qbit+qres) + ((Md*rhoa-Ma*rhod)* g * hbit))/M;
simulation.model.bitFlowRate        = qbit;
simulation.model.chokePressure      = Pc;
simulation.model.chokeFlow          = qc;
simulation.model.mudPumpPressure    = Pp;
simulation.model.qbitDot = qbitDot;
simulation.idealFlow.flowChange = qcDot;
simulation.idealFlow.flow = simulation.controller.flowInput;

simulation.model.massGas            = mg;
simulation.model.qLiquid            = qLiquid;
simulation.model.qGas               = qGas;

simulation.model.rhoa               = rhoa;
simulation.model.betaa              = betaa;
simulation.model.alpha              = alpha;



end


% %%%%%%%
% Define the ODE hydraulic model
% %%%%%%%                   

function [dydt varargout] = defineSystem(t, y, simulation, Vd, betad, Va, VaDot, betag, Ma, Md, Fd, Fa, rhol, rhogIn, rhogOut, rhoaChange, qpump, qback, qres, g, hbit, patm)
%rhol in annulus is (obviously) equal to rhod (density of mud in drill
%string)
% y = [Pp, qbit, Pc, mg, qc]
Pp   = max(patm,y(1));     %Pp can not be less than zero.
qbit = max(0,y(2));     %qbit can not be less than zero. (One-way valve)
Pc   = max(patm,y(3));  %Pc can not be less than atmospheric pressure.
mg   = max(0,y(4));     %mg can not be less than zero.
%y(5)   = max(0,y(5));
%qchoke = y(5);
%qcDot = y(6);

%Calculate volumes
rhogAvg = (rhogIn + rhogOut)/2;
Vag     = mg/rhogAvg;
Val     = Va - Vag;
ml      = rhol*Val;

%The liquid fraction (1=liquid only)
alpha = ml/(ml + mg);

%Calculate density
%rhoa    = (alpha*rhol)*rhoaChange + (1-alpha)*rhogAvg;
rhoa = simulation.model.rhoa;
% betaa   = (betad*betag*Va)/(Va*betag + Vag*(betad-betag)); %Eq. 2-20 in
%Merrit, Hydraulic control systems
betaa   = simulation.model.bulkchange*betad;%0.01 * betad;

%The total mass likness
M = Ma + Md;

%About projections: All the five first states below have a projection. 
%If the state is negative, the derivative can not be
%negative. This is solved by a if-test and a max(0, expression) statement.

dydt = zeros(size(y));

% Choke flow dynamics
qGas = 0;
qLiq = 0;

% omega = simulation.idealFlow.frequency;
% zeta = simulation.idealFlow.dampingRatio;

qchoke = simulation.controller.flowInput;

% A = [0 1; -omega^2 -2*zeta*omega];                       
% B = [0 ; omega^2];    
% 
% if(qchoke > 0)
% dydt(5:6) = A*y(5:6) +B*controlInput;
% else
% dydt(5:6) = A*y(5:6) +B*controlInput;
% dydt(5) = max(0,dydt(5));
% dydt(6) = max(0,dydt(6));
% end

%Mud pump dynamics
if (Pp > patm)
    dydt(1) = (betad/Vd)*(qpump-qbit); 
else
    dydt(1) = max(0,(betad/Vd)*(qpump-qbit));
end

%qbit dynamics
if (qbit> 0)
    dydt(2) = (Pp - Pc - Fd*qbit*abs(qbit) - Fa*(qbit+qres)*abs(qbit+qres) + (rhol - rhoa)*g*hbit )/M; %changed Fa*(qbit)*abs(qbit) to Fa*(qbit+qres)*abs(qbit+qres)
else% (qbit= 0)
    dydt(2) = max(0,(Pp-Pc + (rhol - rhoa)*g*hbit)/M);
end


%Choke pressure dynamics
if(Pc > patm)
    dydt(3) = betaa/Va*(qbit + qback - qchoke + qres-VaDot);
else
    dydt(3) = max(0,betaa/Va*(qback - qchoke + qres-VaDot));
end

%Gas mass dynamics
qres = max(0, qres);% gas does not go back to reservor
if mg > 0
    dydt(4) = qres*rhogIn - qGas*rhogOut;
else
    dydt(4) = 0; % max(0,qres*rhogIn - qGas*rhogOut);
end

%Addition output from define system
if nargout==2
    returnStruct.rhoa   = rhoa;
    returnStruct.betaa  = betaa;
    returnStruct.alpha  = alpha;
    returnStruct.qLiq   = qLiq;
    returnStruct.qGas   = qGas;
    returnStruct.qbitDot = 0;

    varargout(1) = {returnStruct};
end

end