function [ simulation ] = updateCurrentGeometry(simulation)
%UPDATECURRENTGEOMETRY updates LwCurrent and HwCurrent given new lw
%  
%   simulation = updateCurrentGeometry(simulation) updates the current geometry
%   of the well as a response to change in well length lw.
%   
%   Input
%   simulation - superstruct
%        .model.lw - new well length
%        .model.Lw - length profile of well
%        .model.Hw - heigth profile of well
%        .model.Dw - diameter profile of well
%        .model.fW - profile friction in well
%
%   Output
%   simulation - superstruct
%   	.model.LwCurrent - current length position in well
%   	.model.HwCurrent - current heigth in well
%   	.model.DwCurrent - current diameter in well
%   	.model.fWCurrent - current friction

% yvind Stamnes / Torbjrn Pedersen 2008
 

% get well geometry
lw = simulation.model.lw;
Lw = simulation.model.Lw;
Hw = simulation.model.Hw;
Dw = simulation.model.Dw;
fW = simulation.model.fW;

if lw>Lw(end)
    error(['Maximum well length is ' num2str(Lw(end)) ' which is smaller than lw = ' num2str(lw)])
elseif lw<=0
    error(['lw must be larger than zero, lw = ' num2str(lw) ])
end
% find greatest index kl such that Lw(kl)<lw
kl = find(Lw<lw,1,'last');

LwCurrent = zeros(1,kl+1);
HwCurrent = zeros(1,kl+1);

% The current geometry is the same as the static well up to lw
LwCurrent(1:kl)=Lw(1:kl);
HwCurrent(1:kl)=Hw(1:kl);

% last element of LwCurrent is lw
LwCurrent(kl+1)=lw;

% last element of HwCurrent 
HwCurrent(kl+1) = getWellDepth(simulation,simulation.model.lw);

% for diameter and friction its easier
DwCurrent = Dw(1:kl);
fWCurrent = fW(1:kl);

% store them
simulation.model.LwCurrent = LwCurrent;
simulation.model.HwCurrent = HwCurrent;
simulation.model.DwCurrent = DwCurrent;
simulation.model.fWCurrent = fWCurrent;

return