function simulation = updateGeometry(simulation)
%UPDATEGEOMETRY - updates annulus geometry affected parameters
%
%   simulation = updateGeometry(simulation) updates annulus geometry affected
%   parameters La, Aa, Sa and fA. 
%
%   Input
%   simulation - superstruct
%       .model.Ld       - length vector of drilling equipment/string
%       .model.LwCurrent - current length of well
%       .model.DwCurrent - current diameter of well
%       .model.Ddo     - outer diameters of drilling equipment/string
%       .model.Ddi     - inner diameters of drilling equipment/string
%       .model.fWCurrent - current friction in well
%       .model.fDi      - friction factors innside
%       .model.fDo      - friction factors outside
%       .model.rhoa     - density in annulus
%       .model.rhod     - density in drillstring
%       .model.bPos - block position
%
%   Output
%   simulation - superstruct
%       .model.La   - length parameterization vector
%     	.model.Aa   - effective dia^2 
%     	.model.Sa   - surface perimiter
%     	.model.fA   - Friction factor array annulus
%     	.model.Ma   - mass of annulus
%     	.model.Md   - mass of drillstring
%     	.model.Fd   - friction in drillstring
%     	.model.Fa   - friction in annulus
%     	.model.Va   - volume of annulus
%       .model.Vd   - volume of drillstring
%     	.model.hbit - height of bit
% 
% yvind Stamnes / Torbjrn Pedersen 2008

% get properties
Ld = simulation.model.Ld;
LwCurrent = simulation.model.LwCurrent;
DwCurrent = simulation.model.DwCurrent;
Ddo = simulation.model.Ddo;
Ddi = simulation.model.Ddi;
fWCurrent = simulation.model.fWCurrent;
fDi = simulation.model.fDi;
fDo = simulation.model.fDo;
rhoA = simulation.model.rhoa;
rhoD = simulation.model.rhod;
blockPosition = simulation.model.bPos;

% little error check
if (isempty(Ld) || isempty(LwCurrent) || isempty(DwCurrent) || isempty(Ddo) || isempty(fWCurrent) || isempty(fDo))
    error('ERROR: Cannot update geometry when its not defined')
end

% shift Ld so that LdShift(i)=0 corresponds to LwCurrent(j)=0
LdShift = Ld - blockPosition;

if (LdShift(end)>LwCurrent(end))
    disp('LdShift'), LdShift
    disp('blockPos'), blockPosition
    disp('LwCurrent'), LwCurrent
    error('Length of drill pipe in well longer than well')
end

% update La, Aa, Sa and fA
[La, Aa, Sa, fA] = updateLaAaSafA(LdShift,LwCurrent,DwCurrent,Ddo,fWCurrent,fDo);


% update Ma
Ma = updateM(LdShift(end), La, Aa, rhoA);

% update Md
Ad = (pi/4) * Ddi.^2;
Md = updateM(Ld(end), Ld, Ad, rhoD);

% update Fd
Sd = pi * Ddi.^2;
Fd = updateF(Ld(end),fDi,Sd,Ad,Ld,rhoD);

% update Fa
Fa = updateF(LdShift(end), fA,Sa,Aa,La,rhoA);

% update Va

Va = sum(Aa.*diff(La));

% update Vd

Vd = sum((pi/4).*simulation.model.Ddi.^2 .* diff(simulation.model.Ld));

% update hbit
hbit = getWellDepth(simulation,LdShift(end));

% update VaDot
if(simulation.time > 1)
VaDot = Va - simulation.model.Va; % Assumes timestep 1
else 
    VaDot = 0;
end

% update self
simulation.model.La = La;
simulation.model.Aa = Aa;
simulation.model.Sa = Sa;
simulation.model.fA = fA;
simulation.model.Ma = Ma;
simulation.model.Md = Md;
simulation.model.Fd = Fd;
simulation.model.Fa = Fa;
simulation.model.Va = Va;
simulation.model.hbit = hbit;
simulation.model.Vd = Vd;
simulation.model.VaDot = VaDot;

return


%%
function F = updateF(l,f,S,A,L,rho)
% calculates sum_i=1:kl [ f(i)*S(i)/A(i)^3 * ( L(i+1)-L(i) ) * rho/8] with
% L(kl+1)=l, see equations (47), (49), (33), (44) in Kaasa2007

% find out where bit is
kl = find(L<l,1,'last');
F=sum ((f(1:kl).*S(1:kl))./(A(1:kl).^3).*[diff(L(1:kl)) l-L(kl)] * (rho/8));
% B0 = sum(((1/Cd^2)./[diff(L(1:kl)) l-L(kl)])./(A(1:kl).^2));
% B1 = sum(((1/4)*(S(1:kl)))./(A(1:kl).^3));
% F = sum((B0 + f(1:kl)*B1)*(rho/2));

return



%%
function M = updateM(l, L, A, rho)
% calculates sum_i=1:kl [rho * (L(i+1)-L(i))/A(i)] with L(kl+1)=l, see
% equation above (23) in Kaasa2007.


% find out where the bit is
kl = find(L<l,1,'last');
M = sum(rho * [ diff(L(1:kl)) l-L(kl)] ./A(1:kl));

return


%%
function [La, Aa, Sa, fA] = updateLaAaSafA(Ld,LwCurrent,DwCurrent,Ddo,fWarray,fDoarray)

% find indexs of first element of Ld>0
kd=find(Ld>0,1);

% new length parameterization vector
La=0;

% effective dia^2 with new parameterization AreaAnnulus = Da*pi/4
Aa=[];

% Surface perimiter annulus
Sa=[];

% Friction factor array annulus
fA=[];

kw=2; %start from index 2 as index 1 is always zero
while ( kd<= length(Ld)  ) % run to last element of Ld

    % calculate cross sectional area
    Aa = [Aa pi/4*(DwCurrent(kw-1)^2-Ddo(kd-1)^2)];

    % surface perimiter
    Sa = [Sa (DwCurrent(kw-1)^2 + Ddo(kd-1)^2)*pi];

    % friction, average of wall friction and drill string friction
    fA = [fA (fWarray(kw-1)+fDoarray(kd-1))/2];

    if Ld(kd)< LwCurrent(kw)
        % drill string element smallest

        % append drill string length element
        La = [La Ld(kd)];

        % increment
        kd=kd+1;
    elseif LwCurrent(kw)<Ld(kd)
        % current well length element smaller than drill string

        % append well length
        La=[La LwCurrent(kw)];

        % increment
        kw=kw+1;
    else % LwCurrent(kw)==Ld(kd)
        % resolve conflict

        % append lenght (either one will do)
        La = [La LwCurrent(kw)];

        % increment both!
        kw=kw+1;
        kd=kd+1;
    end
end

% append the rest
if kw<=length(LwCurrent)
    % something is left in LwCurrent

    % append it
    La = [La LwCurrent(kw:end)];

    % also append well section below bit
    Aa = [Aa pi/4 *DwCurrent(kw-1:end).^2];

    Sa = [Sa pi*DwCurrent(kw-1:end).^2];

end
return


